//-----------------------------------------------------------------------------
//   File:      eeprom.c
//   Contents:   EEPROM update firmware source.  (Write only)
//
//   indent 3.  NO TABS!
//
//   Copyright (c) 2002 Cypress Semiconductor
//
// $Workfile: eeprom.c $
// $Date: 3/30/05 2:38p $
// $Revision: 2 $
//-----------------------------------------------------------------------------

#include "fx2.h"
#include "fx2regs.h"
#include "syncdly.h"            // SYNCDELAY macro
#include "iic.h"
#include "i2c_sw.h"
#include "gpio.h"
#include "cyanfwserial.h"

#define MSB(word)      (BYTE)(((WORD)(word) >> 8) & 0xff)
#define LSB(word)      (BYTE)((WORD)(word) & 0xff)

/* I2C Control & Status Register (I2CS) 
#define bmSTART      0x80
#define bmSTOP       0x40
#define bmLASTRD     0x20
#define bmID         0x18
#define bmBERR       0x04
#define bmACK        0x02
#define bmDONE       0x01
/* I2C Control Register (I2CTL) 
#define bmSTOPIE     0x02
#define bm400KHZ     0x01*/

void I2CWaitForStop();
bit I2CWaitForAck();
bit I2CWaitForDone();

static void I2CStartAndAddr(BYTE i2cAddr);
extern void WaitForI2CWrite(BYTE i2cAddr);


void I2CStartAndAddr(BYTE i2cAddr)
{
    I2CS = bmSTART;
    //I2DAT = 0x51 << 1;//i2cAddr << 1;
	I2DAT = i2cAddr << 1;
}

void WaitForI2CWrite(BYTE i2cAddr)
{
    I2CWaitForStop();
waitForBusy:
	I2CStartAndAddr(i2cAddr);

    I2CWaitForDone();
    I2CS |= bmSTOP;	//	; Set the STOP bit
    I2CWaitForStop();

    if (!(I2CS & bmACK))  // If no ACK, try again.
       goto waitForBusy;
}

void I2CWaitForStop()
{
   // Data should not be written to I2CS or I2DAT until the STOP bit returns low.
    while (I2CS & bmSTOP);
}

bit I2CWaitForDone()
{
    xdata BYTE i;

    while (!((i = I2CS) & 1));  // Poll the done bit

    if (i & bmBERR)
       return 1;
    else
       return 0;
}

bit I2CWaitForAck()
{
    xdata BYTE i;

    while (!((i = I2CS) & 1));  // Poll the done bit

    if (i & bmBERR)
       return 1;
    else if (!(i & bmACK))
       return 1;
    else
       return 0;
}
#ifdef DBG
bit I2CWrite(BYTE i2cAddr, BYTE sub_type, WORD addr, BYTE len, BYTE *write_dat)
#else
bit I2CWrite(BYTE i2cAddr, WORD addr, BYTE write_dat)
#endif
{
#ifdef DBG
	BYTE i;
#endif
  	I2CWaitForStop();
   
    // write the START bit and i2c device address
    I2CStartAndAddr(i2cAddr);  	
	
	if(I2CWaitForAck())
      return(1);

	if(sub_type != 0) {
	    /* MSB of address in EEPROM */
		if(sub_type == 2) {
			I2DAT = MSB(addr);
		     if(I2CWaitForAck())
		        return(1);
		}     

	    /* LSB of address in EEPROM */
	   	I2DAT = LSB(addr);
		if(I2CWaitForAck())
	        return(1);
	}
#ifdef DBG
	for(i=0; i < len; i++) {
		I2DAT = write_dat[i];
		if(I2CWaitForAck()) {
			I2CS|= bmSTOP;	//	; Set the STOP bit
        	return(1);
		}
	}
#else
	I2DAT = write_dat;
	if(I2CWaitForAck())
        return(1);
#endif
	I2CS|= bmSTOP;	//	; Set the STOP bit
	//WaitForI2CWrite(i2cAddr);
	// success
	return (0);
} 
#ifdef DBG
bit I2CRead(BYTE i2cAddr, BYTE sub_type, WORD addr, BYTE len, BYTE *read_dat) {
#else
bit I2CRead(BYTE i2cAddr, WORD addr, BYTE *read_dat) {
#endif

	
	xdata int value;
#ifdef DBG
	BYTE i;
#endif
	I2CWaitForStop();

	if(sub_type != 0) {		
		/* Write the address first */
   
	   // write the START bit and i2c device address
		I2CStartAndAddr(i2cAddr);

		if(I2CWaitForAck())
	        return(1);

	    /* MSB of address in EEPROM */
		if(sub_type == 2) {
			I2DAT = MSB (addr);
		    if(I2CWaitForAck())
		        return(1);
		}     

	    /* LSB of address in EEPROM */  
    	I2DAT = LSB(addr);

	    if(I2CWaitForAck())
	    {
	        I2CS |= bmSTOP;	//	; Set the STOP bit
	        I2CWaitForStop();
	        return(1);
    	}
	}

    I2CS = bmSTART;

    /* Now read */
    I2DAT = (i2cAddr << 1) | 0x01; //(i2cAddr << 1) | 0x01;
    if(I2CWaitForDone())
        return(1);
#ifdef DBG
    // read dummy byte
	value = I2DAT;
	if(I2CWaitForDone())
		return(1);

	// start reading actual byte	
	for (i = 0; i < len; i++) {
    	read_dat[i] = I2DAT;
		if( i == len - 1) {
			I2CS = bmLASTRD;
		}
    	if(I2CWaitForDone())
    	    return(1);
	}
#else
    // read dummy byte
	for (i = 0; i < len; i++) {
    	value = I2DAT;
		if( i == len - 1) {
			I2CS = bmLASTRD;
		}
    	if(I2CWaitForDone())
    	    return(1);
	}
#endif

    I2CS  = bmSTOP;
    //EEWaitForStop();
     
    //*read_dat = I2DAT;
	return(1);
}

